@@ -145,9 +145,9 @@ __of__(PyObject *inst, PyObject *parent)
 
   UNLESS(r=PyObject_GetAttr(inst, py__of__)) return NULL;
   UNLESS(t=PyTuple_New(1)) goto err;
+  Py_INCREF(parent);
   PyTuple_SET_ITEM(t,0,parent);
   ASSIGN(r,PyObject_CallObject(r,t));
-  PyTuple_SET_ITEM(t,0,NULL);
   Py_DECREF(t);
 
   if (r 
@@ -466,6 +466,14 @@ Wrapper_findattr(Wrapper *self, PyObject *oname,
 	    ASSIGN(r,PyECMethod_New(r,OBJECT(self)));
 	  else if (has__of__(r)) ASSIGN(r,__of__(r,OBJECT(self)));
 
+	  /* DM 2005-04-06: missing return value check leads to
+	     SIGSEGV when "filter" -- check added.
+	     Note that at other places, the filter is applied
+	     before the "__of__". This indicates that more
+	     is wrong (here or elsewhere).
+	  */
+	  if (! r) return r;
+
 	  if (filter)
 	    switch(apply_filter(filter,OBJECT(self),oname,r,extra,orig))
 	      {
@@ -1542,7 +1550,7 @@ initAcquisition(void)
   /* Create the module and add the functions */
   m = Py_InitModule4("Acquisition", methods,
 	   "Provide base classes for acquiring objects\n\n"
-	   "$Id: Acquisition.c,v 1.61.4.1 2003/10/21 12:44:39 andreasjung Exp $\n",
+	   "$Id: Acquisition.c,v 1.61.4.3 2005/06/03 03:27:25 tseaver Exp $\n",
 		     OBJECT(NULL),PYTHON_API_VERSION);
 
   d = PyModule_GetDict(m);
