@@ -12,7 +12,7 @@
 
  ****************************************************************************/
 
-#define BUCKETTEMPLATE_C "$Id: BucketTemplate.c,v 1.54.4.1 2004/04/29 21:51:50 tim_one Exp $\n"
+#define BUCKETTEMPLATE_C "$Id: BucketTemplate.c,v 1.54.4.2 2005/07/05 16:41:33 tim_one Exp $\n"
 
 /* Use BUCKET_SEARCH to find the index at which a key belongs.
  * INDEX    An int lvalue to hold the index i such that KEY belongs at
@@ -688,6 +688,7 @@ Bucket_maxminKey(Bucket *self, PyObject *args, int min)
 {
   PyObject *key=0;
   int rc, offset;
+  int empty_bucket = 1;
 
   if (args && ! PyArg_ParseTuple(args, "|O", &key)) return NULL;
 
@@ -701,6 +702,7 @@ Bucket_maxminKey(Bucket *self, PyObject *args, int min)
       if ((rc = Bucket_findRangeEnd(self, key, min, &offset)) <= 0)
         {
           if (rc < 0) return NULL;
+          empty_bucket = 0;
           goto empty;
         }
     }
@@ -714,7 +716,9 @@ Bucket_maxminKey(Bucket *self, PyObject *args, int min)
   return key;
 
  empty:
-  PyErr_SetString(PyExc_ValueError, "empty bucket");
+  PyErr_SetString(PyExc_ValueError,
+  		  empty_bucket ? "empty bucket" :
+  		                 "no key satisfies the conditions");
   PER_ALLOW_DEACTIVATION(self);
   PER_ACCESSED(self);
   return NULL;
