@@ -17,7 +17,7 @@ Page Template-specific implementation of TALES, with handlers
 for Python expressions, string literals, and paths.
 """
 
-__version__='$Revision: 1.43.44.4 $'[11:-2]
+__version__='$Revision: 1.43.44.5 $'[11:-2]
 
 import re, sys
 from TALES import Engine, CompilerError, _valid_name, NAME_RE, \
@@ -308,7 +308,7 @@ def restrictedTraverse(object, path, securityManager,
                 raise Unauthorized, name
         else:
             # Try an attribute.
-            o = guarded_getattr(object, name, M)
+            o = guarded_getattr(object, str(name), M) # failed on u'aq_parent'
             if o is M:
                 # Try an item.
                 try:
