@@ -31,7 +31,7 @@ that allows one to simply make a single web request.
 The module also provides a command-line interface for calling objects.
 
 """
-__version__='$Revision: 1.45.66.1 $'[11:-2]
+__version__='$Revision: 1.45.66.2 $'[11:-2]
 
 import sys, re, socket, mimetools
 from httplib import HTTP
@@ -141,8 +141,7 @@ class Function:
                 )
 
         try:
-            h=HTTP()
-            h.connect(self.host, self.port)
+            h=HTTP(self.host, self.port)
             h.putrequest(method, self.rurl)
             for hn,hv in headers.items():
                 h.putheader(translate(hn,dashtrans),hv)
