@@ -11,7 +11,7 @@
 #
 ##############################################################################
 
-__version__='$Revision: 1.90.2.8 $'[11:-2]
+__version__='$Revision: 1.90.2.9 $'[11:-2]
 
 import re, sys, os,  urllib, time, random, cgi, codecs
 from types import StringType, UnicodeType
@@ -386,7 +386,7 @@ class HTTPRequest(BaseRequest):
         if not hasattr(fs,'list') or fs.list is None:
             # Hm, maybe it's an XML-RPC
             if (fs.headers.has_key('content-type') and
-                fs.headers['content-type'] == 'text/xml' and
+                'text/xml' in fs.headers['content-type'] and
                 method == 'POST'):
                 # Ye haaa, XML-RPC!
                 global xmlrpc
